module SearchyJson
  class Query
    attr_reader :text

    NEGATIVE = /\A-/.freeze
    START_EXTRACT = /\A("|')/.freeze
    END_EXTRACT = /("|')\Z/.freeze

    def initialize(text)
      @text = text
    end

    def negative_keywords
      keywords_by_type[:negative]
    end

    def positive_keywords
      keywords_by_type[:positive]
    end

    def exact_matches?
      @exact_matches ||= !(text =~ START_EXTRACT && text =~ END_EXTRACT).nil?
    end

    private

    def keywords_by_type
      @keywords_by_type ||= exact_keywords_by_type
    end

    def exact_keywords_by_type
      negative = []
      positive = []

      keywords.each do |word|
        negative?(word) ? negative.push(word.gsub(NEGATIVE, '')) : positive.push(word)
      end

      { negative: negative, positive: positive }
    end

    def negative?(keyword)
      keyword =~ NEGATIVE
    end

    def keywords
      @keywords ||= text.gsub(START_EXTRACT, '').gsub(END_EXTRACT, '').upcase.split(' ')
    end
  end
end
